@@ -1,4 +1,4 @@
-# Copyright 2009 Canonical Ltd.
+# Copyright 2009-2018 Canonical Ltd.
 
 # This file is part of lazr.restfulclient.
 #
@@ -28,12 +28,9 @@ import os
 import platform
 import stat
 import socket
-# Work around relative import behavior.  The below is equivalent to
-# from oauth import oauth
-oauth = __import__('oauth.oauth', {}).oauth
-(OAuthConsumer, OAuthRequest, OAuthSignatureMethod_PLAINTEXT,
- OAuthToken) = (oauth.OAuthConsumer, oauth.OAuthRequest,
-                oauth.OAuthSignatureMethod_PLAINTEXT, oauth.OAuthToken)
+
+from oauthlib import oauth1
+import six
 
 from lazr.restfulclient.authorize import HttpAuthorizer
 from lazr.restfulclient.errors import CredentialsFileError
@@ -50,12 +47,9 @@ __all__ = [
 CREDENTIALS_FILE_VERSION = '1'
 
 
-# These two classes are provided for convenience (so applications
-# don't need to import from oauth.oauth), and to provide
-# lazr.restfulclient-specific enhancements like a default argument for
-# secret (which shouldn't be used), and an application name separate
-# from the consumer key.
-class Consumer(oauth.OAuthConsumer):
+# For compatibility, Consumer and AccessToken are defined using terminology
+# from the older oauth library rather than the newer oauthlib.
+class Consumer:
     """An OAuth consumer (application)."""
 
     def __init__(self, key, secret='', application_name=None):
@@ -68,18 +62,29 @@ class Consumer(oauth.OAuthConsumer):
             from the consumer key. If present, this will be used in
             the User-Agent header.
         """
-        OAuthConsumer.__init__(self, key, secret)
+        self.key = key
+        self.secret = secret
         self.application_name = application_name
 
 
-class AccessToken(oauth.OAuthToken):
+class AccessToken:
     """An OAuth access token."""
 
     def __init__(self, key, secret='', context=None):
-        OAuthToken.__init__(self, key, secret)
+        self.key = key
+        self.secret = secret
         self.context = context
 
 
+class TruthyString(six.text_type):
+    """A Unicode string which is always true."""
+
+    def __bool__(self):
+        return True
+
+    __nonzero__ = __bool__
+
+
 class SystemWideConsumer(Consumer):
     """A consumer associated with the logged-in user rather than an app.
 
@@ -108,12 +113,11 @@ class SystemWideConsumer(Consumer):
         hostname. It does not identify the active user.
         """
         try:
-            distname, version, release_id = platform.linux_distribution()
+            import distro
+            distname = distro.name()
         except Exception:
-            # This can happen in pre-2.6 versions of Python. We don't
-            # use platform.dist() because it looks like old versions
-            # of dist can give inconsistent answers on a single
-            # computer.
+            # This can happen due to various kinds of failures with the data
+            # sources used by the distro module.
             distname = ''
         if distname == '':
             distname = platform.system() # (eg. "Windows")
@@ -240,9 +244,18 @@ class OAuthAuthorizer(HttpAuthorizer):
 
     def authorizeRequest(self, absolute_uri, method, body, headers):
         """Sign a request with OAuth credentials."""
-        oauth_request = OAuthRequest.from_consumer_and_token(
-            self.consumer, self.access_token, http_url=absolute_uri)
-        oauth_request.sign_request(
-            OAuthSignatureMethod_PLAINTEXT(),
-            self.consumer, self.access_token)
-        headers.update(oauth_request.to_header(self.oauth_realm))
+        client = oauth1.Client(
+            self.consumer.key,
+            client_secret=self.consumer.secret,
+            resource_owner_key=TruthyString(self.access_token.key or ''),
+            resource_owner_secret=self.access_token.secret,
+            signature_method=oauth1.SIGNATURE_PLAINTEXT,
+            realm=self.oauth_realm)
+        # The older oauth library (which may still be used on the server)
+        # requires the oauth_token parameter to be present and will fail
+        # authentication if it isn't.  This hack forces it to be present
+        # even if its value is the empty string.
+        client.resource_owner_key = TruthyString(client.resource_owner_key)
+        _, signed_headers, _ = client.sign(absolute_uri)
+        for key, value in signed_headers.items():
+            headers[key.encode('UTF-8')] = value.encode('UTF-8')
